import contextlib
import logging
import datetime
from typing import List

import requests
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

default_logger = logging.getLogger()


class Job:
    def __init__(self, raw_json):

        self.raw_json = raw_json
        self.job_id = raw_json['jobId']
        self.name = raw_json['name']
        self.type = raw_json['type']

        # Possible: Wait Condition, Ended OK, Executing, Wait Resource, Wait User, Ended Not OK, Unknown, Not in AJF
        self.status = raw_json.get('status', 'Unknown')
        self.number_of_runs = raw_json.get('numberOfRuns', 0)

        self.start_time = None
        if raw_json['startTime']:
            self.start_time = datetime.datetime.strptime(raw_json['startTime'], '%Y%m%d%H%M%S')

        self.end_time = None
        if raw_json['endTime']:
            self.end_time = datetime.datetime.strptime(raw_json['endTime'], '%Y%m%d%H%M%S')

    def __str__(self):
        return f'Job({self.__dict__})'

    def __repr__(self):
        return self.__str__()


class JobStatus:
    def __init__(self, raw_json):
        self.returned = raw_json['returned']
        self.total = raw_json['total']
        self.jobs: List[Job] = [Job(job_json) for job_json in raw_json['statuses']]

        self.items_per_page = raw_json.get('itemsPerPage', None)
        self.monitor_page_url = raw_json.get('monitorPageURI', None)
        self.next_uri = raw_json.get('nextURI', None)
        self.prev_uri = raw_json.get('prevURI', None)
        self.sort = raw_json.get('sort', None)
        self.start_index = raw_json.get('startIndex', None)

    def __str__(self):
        return f'JobStatus(returned: {self.returned}, total: {self.total}, items_per_page: {self.items_per_page}, next_uri: {self.next_uri})'

    def __repr__(self):
        return self.__str__()


class ControlM:
    def __init__(self, base_url: str, username: str, password: str, logger=default_logger):
        self._base_url = base_url
        if base_url.endswith('/'):
            self._base_url = base_url[:-1]

        self._auth_details = {'username': username, 'password': password}
        self.logger = logger

    def _make_request(self, path, method='GET', json=None, headers=None, params=None):
        url = f'{self._base_url}{path}'
        self.logger.info(f'Executing {method} to {url}')
        r = requests.request(method, url, json=json, headers=headers, verify=False, params=params, timeout=30)
        self.logger.info(f'{url}: {r}')
        if r.status_code >= 400:
            raise Exception(f'Error executing {url}: {r}: {r.text}')
        try:
            return r.json()
        except:
            return r.text

    @contextlib.contextmanager
    def login(self) -> str:
        token = self._make_request('/automation-api/session/login', method='POST', json=self._auth_details)['token']
        yield token
        self._make_auth_request('/automation-api/session/logout', method='POST', json=self._auth_details, token=token)

    def _make_auth_request(self, path, method='GET', params=None, json=None, token=None):
        if token is None:
            token = self.login()
        auth_header = {'Authorization': f'Bearer {token}'}
        return self._make_request(path, method=method, json=json, headers=auth_header, params=params)

    def job_status(self, token: str) -> JobStatus:
        path = '/automation-api/run/jobs/status'
        params = {
            'limit': 10000
        }
        return JobStatus(self._make_auth_request(path, token=token, params=params))

    def config_servers(self, token: str):
        path = '/automation-api/config/servers'
        return self._make_auth_request(path, token=token)

    def config_server_agents(self, server, token: str):
        path = f'/automation-api/config/server/{server}/agents'
        return self._make_auth_request(path, token=token)

    def config_server_params(self, server, token: str):
        path = f'/automation-api/config/server/{server}/params'
        return self._make_auth_request(path, token=token)

    def resources(self, token: str):
        path = '/automation-api/run/resources'
        return self._make_auth_request(path, token=token)

    def deploy_jobs(self, server, folder, token: str, job_format='json'):
        path = '/automation-api/deploy/jobs '
        return self._make_auth_request(path, params={'format': job_format, 'ctm': server, 'folder': folder}, token=token)

    def folders(self, token: str):
        path = '/automation-api/deploy/deployed/*/*'
        return self._make_auth_request(path, token=token)

    def samples(self, token: str):
        path = '/automation-api/samples'
        return self._make_auth_request(path, token=token)


def main():
    import json
    from collections import Counter

    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    logger.addHandler(ch)

    with open('properties.json', 'r') as f:
        config = json.load(f)
        c = ControlM(config['url'], config['user'], config['pass'], logger=logger)

    with c.login() as token:
        print(c.job_status(token))


if __name__ == '__main__':
    main()

